import bluej.extensions.BlueJ;
import bluej.extensions.Extension;
import bluej.extensions.PreferenceGenerator;
import bluej.extensions.event.CompileEvent;
import bluej.extensions.event.CompileListener;
import bluej.extensions.event.PackageEvent;
import bluej.extensions.event.PackageListener;
import bluej.utility.Debug;
import java.io.File;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.util.Properties;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Start of error logging BlueJ extension
 *
 * @author Warren Toomey
 */
public class ErrLogExtension extends Extension implements PackageListener {
    public static final String URL_DEFAULT =
        			"http://minnie.tuhs.org/cgi-bin/bluejerr";
    public static final String URL_LABEL = "URL";

    /**
     * Initialises the extension by setting the generator for menu items
     */
    public void startup ( BlueJ bluej ) {
        Preferences myPreferences = new Preferences ( bluej );
        bluej.setPreferenceGenerator ( myPreferences );
        ErrCompileListener l= new ErrCompileListener(
            System.currentTimeMillis() / 1000,
            System.getProperty("user.name"));
        bluej.addCompileListener(l);
    }

    /**
     * This method must decide if this Extension is compatible with the
     * current release of the BlueJ Extensions API
     */
    public boolean isCompatible ( ) {
        return true;
    }

    /**
     * Returns the version number of this extension
     */
    public String getVersion ( ) {
        return ( "2011_Oct_30" );
    }

    /**
     * Returns the user-visible name of this extension
     */
    public String getName ( ) {
        return ( "Error Logging Extension" );
    }

    /**
     * Returns a simple description of the extension
     */
    public String getDescription ( ) {
        return ( "Extension for logging BlueJ compile errors" );
    }

    /**
     * Returns a URL where you can find info on this extension.
     * TODO: Set a proper URL
     */
    public URL getURL ( ) {
        try {
            return new URL ( "http://minnie.tuhs.org/Programs/BlueJErrors/" );
        } catch ( MalformedURLException e ) {
        }
        return null;
    }

    @Override
    public void packageOpened ( PackageEvent event ) {
    }

    @Override
    public void packageClosing ( PackageEvent event ) {
    }




    class ErrCompileListener implements CompileListener {
        private long sessionNumber;
        private String username = "unknown";

        public ErrCompileListener(long s, String u) {
            sessionNumber= s;
            if (u != null) {
                try {
                    MessageDigest md = MessageDigest.getInstance("MD5");
                    byte [] result = md.digest( u.getBytes() );
                    String str = new BigInteger(1, result).toString(16);
                    username = str;
                } catch (Exception e) { }
            }
        }

        public void sendCompileEvent(CompileEvent event, String m) {
            final String useProxiesProperty = "java.net.useSystemProxies";
            String oldProxySetting = "false";   // Documented default value
            int type= event.getEvent();
            String msg= (m != null) ? m : event.getErrorMessage();
            int linenum = event.getErrorLineNumber();
            File [] files = event.getFiles();
            String fileName = "unknown.java";

            Debug.message("Error " + event);
            Debug.message("-> " + type + " " + msg);

            if (files.length > 0) {
                fileName= files[0].getName();
            }

            try {
                // Attempt to use local proxy settings to avoid any
                // firewalls, just for the rest of this method.
                oldProxySetting = System.getProperty(useProxiesProperty,
                                                     oldProxySetting);
                System.setProperty(useProxiesProperty,"true");

                URL url = new URL(URL_DEFAULT +
                 "?session=" + URLEncoder.encode("" + sessionNumber, "UTF-8") +
                 "&user=" + URLEncoder.encode(username, "UTF-8") +
                 "&type=" + URLEncoder.encode("" + type, "UTF-8") +
                 "&line=" + URLEncoder.encode("" + linenum, "UTF-8") +
                 "&file=" + URLEncoder.encode(fileName, "UTF-8") +
                 "&message=" + URLEncoder.encode(msg, "UTF-8")
                );
                HttpURLConnection conn =
                    (HttpURLConnection) url.openConnection();
                conn.connect();
                int rc = conn.getResponseCode();
                conn.disconnect();

                if (rc != 200)
                    Debug.reportError(
                        "Update stats failed, HTTP response code: " + rc);
            } catch (Exception ex) {
                Debug.reportError("Update stats failed: " +
                                  ex.getClass().getName() +
                                  ": " + ex.getMessage());
            } finally {
                System.setProperty(useProxiesProperty, oldProxySetting);
            }
        }

        public void compileError(CompileEvent event) {
            sendCompileEvent(event, null);
        }

        public void compileFailed(CompileEvent event) {
        }

        public void compileStarted(CompileEvent event) {
            sendCompileEvent(event, "Compile Started");
        }

        public void compileSucceeded(CompileEvent event) {
        }

        public void compileWarning(CompileEvent event) {
        }
    }


    class Preferences implements PreferenceGenerator {
        private JPanel panel;
        private JTextField url;
        private BlueJ bluej;

        // Construct the panel, and initialise it from any stored values
        public Preferences ( BlueJ bluej ) {
            this.bluej = bluej;

            this.panel = new JPanel ( );
            this.panel.setBorder (
                BorderFactory.createEmptyBorder ( 10, 10, 10, 10 ) );
            this.panel.setLayout ( new GridLayout ( 0, 2 ) );

            this.panel.add ( new JLabel ( "Error logging server:" ) );
            this.url = new JTextField ( 40 );
            this.panel.add ( url );

            // Load the default values
            this.loadValues ( );
        }

        public JPanel getPanel ( ) {
            return this.panel;
        }

        public void saveValues ( ) {
            // Save all preferences to the BlueJ config file
            ( new Thread ( ) {
                public void run ( ) {
                    Preferences.this.bluej.setExtensionPropertyString (
                        ErrLogExtension.URL_LABEL,
                        Preferences.this.url.getText ( ) );
                }
            } ).start();
        }

        public void loadValues ( ) {
            // Load values from settings with "sensible" defaults
            ( new Thread ( ) {
                public void run ( ) {
                    Preferences.this.url.setText (
                        Preferences.this.bluej.getExtensionPropertyString (
                            ErrLogExtension.URL_LABEL,
			    ErrLogExtension.URL_DEFAULT ) );
                }
            } ).start();
        }
    }
}
